from .__about__ import (
    __version__,
    __title__,
    __summary__
)

from .singleton import (  # noqa: F401
    PySingleton,
    PyMemoized
)

__all__ = [
    "__version__",
    "__title__",
    "__summary__"
]



import os

from weakref import WeakValueDictionary


class PySingleton(type):
    _instances = None

    def __call__(cls, *args, **kwargs):
        if not cls._instances:
            cls._initialize()
        instance = cls._instances.get(cls)
        if not instance:
            instance = super(
                PySingleton, cls
            ).__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return instance

    def _initialize(cls):
        use_weakref = getattr(cls, "_PYSINGLETON_WEAKREF", None)
        env_var = f"{cls.__name__}_WEAKREF"
        if use_weakref is None:
            use_weakref = os.getenv(env_var)
            use_weakref = True if use_weakref == "1" else False
        if use_weakref:
            cls._instances = WeakValueDictionary()
        else:
            cls._instances = {}


class PyMemoized(type):
    _instances = None

    def __call__(cls, memoized_key, *args, **kwargs):
        if not cls._instances:
            cls._initialize()
        instance = cls._instances.get((cls, memoized_key))

        if instance is None:
            instance = super().__call__(memoized_key, *args, **kwargs)
            cls._instances[(cls, memoized_key)] = instance
        return instance

    def _initialize(cls):
        use_weakref = getattr(cls, "_PYMEMOIZED_WEAKREF", None)
        env_var = f"{cls.__name__}_WEAKREF"
        if use_weakref is None:
            use_weakref = os.getenv(env_var)
            use_weakref = True if use_weakref == "1" else False
        if use_weakref:
            cls._instances = WeakValueDictionary()
        else:
            cls._instances = {}



from setuptools import (
    setup,
    find_packages
)

about = {}
with open("pysingleton/__about__.py") as fp:
    exec(fp.read(), about)

with open("README.md", "r") as fp:
    long_description = fp.read()

setup(name='python-singleton-metaclasses',
      version=about["__version__"],
      description=about["__summary__"],
      long_description=long_description,
      long_description_content_type="text/markdown",
      author="Zachary Cutlip",
      author_email="uid000@gmail.com",
      url="https://github.com/zcutlip/py-singleton-metaclasses",
      license="MIT",
      packages=find_packages(),
      python_requires='>=3.7',
      install_requires=[],
      package_data={'pysingleton': ['config/*']},
      )
